Attribute VB_Name = "modConfig"
'    Blue Ice HTML Editor
'    Copyright (C) 1999  Joel Dueck
'
'    This program is free software; you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation; either version 2 of the License, or
'    (at your option) any later version.
'
'    This program is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program; if not, write to the Free Software
'    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'

Option Explicit

Public ImgDrive$, ImgPath$
Public UcaseTags As Boolean
Public TargetBrowserPath$
Public LastSaveDir$
Public MRUFiles As New Collection
Public DocMaximized As Boolean, ShowSplash As Boolean
Public StartNewFile As Boolean
Public AutosaveInterval As Integer
Public GStartColor As Long, GEndColor As Long
Public FastStart As Boolean

Const DEF_GSTART = 0
Const DEF_GEND = 12615680

Public dFont$, dFontBold As Boolean, dFontItalic As Boolean, _
    dFontUnderline As Boolean, dFontSize%

Public Const REG_KEY = "SOFTWARE\JoelDueck\BlueIce"

Public Function AppPath() As String
Dim A$
    A$ = App.Path
    AppPath = A$ & IIf(Right$(A$, 1) = "\", "", "\")
End Function

Public Sub LoadOptions()
Dim creg As New CRegistry
    creg.root = HkeyLocalMachine
    creg.key = REG_KEY
    
    With creg
        ImgDrive = .GetValue("ImgDrive")
        ImgPath = .GetValue("ImgPath")
        UcaseTags = GetRegBool(creg, "UCaseTags", False)
        TargetBrowserPath = .GetValue("Targetbrowser")
        LastSaveDir = .GetValue("LastSaveDir")
        DocMaximized = GetRegBool(creg, "DocMaximized", True)
        StartNewFile = GetRegBool(creg, "StartNewFile", False)
        ShowSplash = GetRegBool(creg, "ShowSplash", True)
        AutosaveInterval = GetRegInt(creg, "SaveInterval", 3)
        GStartColor = GetRegLong(creg, "GradientStart", DEF_GSTART)
        GEndColor = GetRegLong(creg, "GradientEnd", DEF_GEND)
        FastStart = GetRegBool(creg, "FastStart", True)
        
        ' This section loads the Most Recently Used
        ' files list
        '
        Dim F$, I%
        For I = 1 To 4
            F$ = .GetValue("MRUFile" & CStr(I))
            If Len(F$) > 0 Then
                MRUFiles.Add F$, F$
            End If
        Next I
        
        UpdateMRU
    End With
End Sub

' The window state loading and saving has been isolated
' from the other options in order to allow easy IO of said
' other options
'
Public Sub LoadCoords()
Dim WS%
Dim W%, H%, T%, L%
Dim creg As New CRegistry
    creg.root = HkeyLocalMachine
    creg.key = REG_KEY
    
    With frmMain
        WS% = GetRegInt(creg, "WindowState", 0)
        If WS = vbMinimized Then WS = vbNormal
        
        W = GetRegInt(creg, "Width", 6495)
        H = GetRegInt(creg, "Height", 6495)
        
        .Width = W
        .Height = H
    
        T = GetRegInt(creg, "Top", (Screen.Height - .Height) / 2)
        L = GetRegInt(creg, "Left", (Screen.Width - .Width) / 2)
        
        .Left = L
        .Top = T
        
        .WindowState = WS
    End With
End Sub

Public Sub SaveCoords()
Dim creg As New CRegistry
    creg.root = HkeyLocalMachine
    creg.key = REG_KEY
    
    With frmMain
        creg.SetValue "WindowState", CStr(.WindowState)
        If .WindowState <> vbNormal Then Exit Sub
        creg.SetValue "Width", CStr(.Width)
        creg.SetValue "Height", CStr(.Height)
        creg.SetValue "Top", CStr(.Top)
        creg.SetValue "Left", CStr(.Left)
    End With
End Sub

Public Function GetRegInt(CR As CRegistry, Value As String, Default As Integer) As Integer
Dim T$
    T$ = CR.GetValue(Value)
    If Len(T$) = 0 Then
        GetRegInt = Default
    Else
        GetRegInt = CInt(T$)
    End If
End Function

Public Function GetRegLong(CR As CRegistry, Value As String, Default As Long)
Dim T$
    T$ = CR.GetValue(Value)
    If Len(T$) = 0 Then
        GetRegLong = Default
    Else
        GetRegLong = CLng(T$)
    End If
End Function

Public Function GetRegBool(CR As CRegistry, Value As String, Default As Boolean) As Boolean
Dim T$
    T$ = CR.GetValue(Value)
    If Len(T$) = 0 Then
        GetRegBool = Default
    Else
        GetRegBool = CBool(T$)
    End If
End Function

Public Function GetRegStr(CR As CRegistry, Value As String, Default As String) As String
Dim T$
    T$ = CR.GetValue(Value)
    GetRegStr = IIf(Len(T$) = 0, Default, T$)
End Function

' Protects against errors resulting from trying to
' save zero-length strings to the Registry
'
Public Sub SaveString(CR As CRegistry, sKeyName As Variant, sValue As String)
    If Len(sValue) > 0 Then
        CR.SetValue sKeyName, sValue
    End If
End Sub

Public Sub SaveOptions()
Dim creg As New CRegistry, I%, F$

    creg.root = HkeyLocalMachine
    creg.key = REG_KEY
    creg.AutoCreate = True
        
    SaveString creg, "ImgPath", ImgPath
    SaveString creg, "ImgDrive", ImgDrive
    SaveString creg, "Targetbrowser", TargetBrowserPath
    SaveString creg, "LastSaveDir", LastSaveDir
    creg.SetValue "DocMaximized", CStr(DocMaximized)
    creg.SetValue "ShowSplash", CStr(ShowSplash)
    creg.SetValue "StartNewFile", CStr(StartNewFile)
    creg.SetValue "UCaseTags", CStr(UcaseTags)
    creg.SetValue "SaveInterval", CStr(AutosaveInterval)
    creg.SetValue "GradientStart", CStr(GStartColor)
    creg.SetValue "GradientEnd", CStr(GEndColor)
    creg.SetValue "FastStart", CStr(FastStart)
    
    For I = 1 To MRUFiles.Count
        SaveString creg, "MRUFile" & CStr(I), MRUFiles(I)
    Next I
End Sub

Public Sub SaveFontSettings()
Dim creg As New CRegistry, I%, F$

    creg.root = HkeyLocalMachine
    creg.key = REG_KEY
    creg.AutoCreate = True
    
    creg.SetValue "FontName", dFont
    creg.SetValue "FontSize", CStr(dFontSize)
    creg.SetValue "FontBold", CStr(dFontBold)
    creg.SetValue "FontItalic", CStr(dFontItalic)
    creg.SetValue "FontUnderline", CStr(dFontUnderline)
End Sub

Public Sub LoadFontSettings()
Dim creg As New CRegistry, I%, F$

    creg.root = HkeyLocalMachine
    creg.key = REG_KEY
    creg.AutoCreate = True
    
    dFont = GetRegStr(creg, "FontName", "Verdana")
    dFontBold = GetRegBool(creg, "FontBold", False)
    dFontItalic = GetRegBool(creg, "FontItalic", False)
    dFontSize = GetRegInt(creg, "FontSize", 10)
    dFontUnderline = GetRegBool(creg, "FontUnderline", False)
End Sub
