Attribute VB_Name = "modUtil"
'    Blue Ice HTML Editor
'    Copyright (C) 1999  Joel Dueck
'
'    This program is free software; you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation; either version 2 of the License, or
'    (at your option) any later version.
'
'    This program is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program; if not, write to the Free Software
'    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
'

Option Explicit

' API declares
'
' These used for allowing the user to drag a form using
' my custom gradient titlebar.
Public Declare Function ReleaseCapture Lib "user32" () As Long
Public Declare Function SendMessage Lib "user32" _
    Alias "SendMessageA" (ByVal hWnd As Long, _
    ByVal wMsg As Long, ByVal wParam As Long, _
    lParam As Any) As Long
    
Declare Function WinHelp Lib "user32" Alias "WinHelpA" (ByVal hWnd As Long, _
    ByVal lpHelpFile As String, ByVal wCommand As Long, _
    ByVal dwData As Long) As Long

Public Const WM_SYSCOMMAND = &H112
Public Const MOUSE_MOVE = &HF012

' These consts used for the multi-level undo capability
'
Public Const WM_USER = &H400
Public Const EM_HIDESELECTION = WM_USER + 63

' These collections hold lists of previously used filenames
' for use in autocomplete of links
'
Public LinkHistory As New Collection
'Public ImgHistory As New Collection

Public CustomApps As New Collection
Public Templates As New Collection
Public CodeClips As New Collection
Public MRUDirs As New Collection

' Keep track of temporary files
Public TempFiles As New Collection

' A shortcut to Panel 1 in the status bar of the main
' window, set by the main form when its loaded
'
Public Status As Panel

' This integer ensures unique doc IDs within a session; it
' is incremented for each new document, see
' frmDoc:Form_Load event.
'
Public FreeNum As Integer

' Last search text
Public LastSearch As String

Public Enum kFilePart
    kfpDrive
    kfpPath
    kfpDrivePath
    kfpExtension
    kfpFilename
    kfpPathFile
End Enum

' Function for getting parts of a filename
' Handles local filenames ONLY!
'
Public Function GetFilePart(Filename$, FilePart As kFilePart) As String
Static LastF$
Static Dr$, P$, E$, F$

Dim StartP%, LastS%, EndP%
Dim StartE%, EndE%

    If Filename <> LastF Then
            
        P$ = ""
        Dr$ = ""
        E$ = ""
        
        If Filename Like "[a-zA-Z]:\*" Then
            Dr = Left$(Filename, 2)
        End If
        
        If Filename Like "*\*" Then
            If Len(Dr$) > 0 Then
                StartP = 3
                LastS = 1
            Else
                StartP = 1
                LastS = 1
            End If
            
            Do
                LastS = InStr(LastS, Filename, "\")
                If LastS = 0 Then
                    P$ = Mid$(Filename, StartP, (EndP - StartP) + 1)
                Else
                    EndP = LastS
                    LastS = LastS + 1
                End If
            Loop Until Len(P$) > 0
        End If
        
        If Filename Like "*.*" Then
            LastS = 1
            Do
                LastS = InStr(LastS, Filename, ".")
                If LastS = 0 Then
                    E$ = Right$(Filename, Len(Filename) - EndE)
                Else
                    EndE = LastS
                    LastS = LastS + 1
                End If
            Loop Until Len(E$) > 0
        End If
        
        If Len(P) = 0 And Len(Dr) = 0 Then
            F$ = Filename
        Else
            F$ = Right$(Filename, Len(Filename) - EndP)
        End If
    End If
    
    Select Case FilePart
        Case kfpDrive
            GetFilePart = Dr$
        Case kfpDrivePath
            GetFilePart = Dr$ & P$
        Case kfpPath
            GetFilePart = P$
        Case kfpPathFile
            GetFilePart = P$ & F$
        Case kfpExtension
            GetFilePart = E$
        Case kfpFilename
            GetFilePart = F$
    End Select
    LastF = Filename
End Function

Public Sub LoadHistory()
Dim F$, N$, I$, cClip As New CCodeClip
    FileFillCol LinkHistory, AppPath & "links.his"
    FileFillCol MRUDirs, AppPath & "dirs.his"
    FileFillCol TempFiles, AppPath & "tempdoc.his"
    FileFillCol Templates, AppPath & "templates.dat"
    LoadCodeClips
    LoadApps
End Sub

Public Sub SaveHistory()
Dim cClip As New CCodeClip, I%
    FileDumpCol Templates, AppPath & "templates.dat"
    FileDumpCol LinkHistory, AppPath & "links.his"
    FileDumpCol MRUDirs, AppPath & "dirs.his"
    FileDumpCol TempFiles, AppPath & "tempdoc.his"
End Sub

Public Sub LoadCodeClips()
Dim I$, cClip As New CCodeClip
    If Len(Dir(AppPath & "clips.dat")) = 0 Then Exit Sub

    Open AppPath & "clips.dat" For Input As #1
    While Not EOF(1)
        Line Input #1, I$
        If I$ = "``**``**_&" Then
            CodeClips.Add cClip, cClip.Name
            Set cClip = New CCodeClip
        Else
            If Len(cClip.Name) = 0 Then
                cClip.Name = I$
            Else
                cClip.Code = cClip.Code & I$
            End If
        End If
    Wend
    Close #1
End Sub

Public Sub SaveCodeClips()
Dim Clip As CCodeClip
    Open AppPath & "clips.dat" For Output As #1
    For Each Clip In CodeClips
        Print #1, Clip.Name
        Print #1, Clip.Code
        Print #1, "``**``**_&"
    Next Clip
    Close #1
End Sub

' FileFillCol and FileDumpCol provide simple line-based
' IO for saving collections to text files. Each item
' in a collection is stored as a line.
'
Public Sub FileFillCol(C As Collection, Filename$)
Dim A$
    On Error GoTo NoFile
    Open Filename For Input As #1
    
    While Not EOF(1)
        Line Input #1, A$
        If Len(A$) > 0 Then C.Add A$, A$
    Wend
    Close #1
NoFile:
End Sub

Public Sub FileDumpCol(C As Collection, Filename$)
Dim I%
    Open Filename For Output As #1
    
    For I = 1 To C.Count
        If Len(C.item(I)) > 0 Then Print #1, C.item(I)
    Next I
    
    Close #1
End Sub

' Checks for the existence of a string in a list
Public Function InList(L As Variant, S As String) As Boolean
Dim I%, C As New Collection, T$
    
    T$ = TypeName(L)
    
    ' If we have a combobox or listbox here, copy it over
    ' to a collection. This way we can handle all kinds
    ' of requests.
    '
    If T$ = "ComboBox" Or T$ = "ListBox" Then
        If L.ListCount = 0 Then
            InList = False
            Exit Function
        End If
        
        For I = 0 To L.ListCount - 1
            C.Add L.List(I)
        Next I
    ElseIf T$ = "Collection" Then
        Set C = L
    Else
        InList = False
        Exit Function
    End If
    
    If C.Count = 0 Then
        InList = False
        Exit Function
    Else
        ' Do a non-case-sensitive search
        For I = 1 To C.Count
            If (S = C(I)) Or ((Len(S) = Len(C(I))) And _
                InStr(1, S, C(I), 1)) Then
                    InList = True
                    Exit Function
            End If
        Next I
    End If
    
    InList = False
End Function

'  This function lets the computer make an informed
' guess at what you want while you are typing it, based
' on some kind of list that contains values you have used
' in the past.  Used in image link, hyperlink, and other places.
'
Public Sub AutoComplete(L As ComboBox)
Dim strText As String
Dim strLength As Integer
Dim I%

    strText = LCase$(L.Text)
    strLength = Len(strText)
    
    If strLength > 0 Then
        For I = 0 To L.ListCount - 1
            If strText = LCase$(Left$(L.List(I), strLength)) Then
                ' We found something
                If Len(L.List(I)) > strLength Then
                    L.SelText = Mid$(L.List(I), strLength + 1)
                    L.SelStart = strLength
                    L.SelLength = Len(Mid$(L.List(I), strLength + 1))
                End If
                Exit For
            End If
        Next I
    End If
End Sub

' Updates the MRU menu list using a collection
'
Public Sub UpdateMRU()
Dim I%
    If MRUFiles.Count > 0 Then
        
        ' Remove the bottom item if necessary
        If MRUFiles.Count > 4 Then
            While MRUFiles.Count > 4
                MRUFiles.Remove 5
            Wend
        End If
        
        ' Set menu items
        For I = 1 To MRUFiles.Count
            frmMain.mnuMRU(I - 1).Caption = "&" & _
                CStr(I) & " " & MRUFiles(I)
            frmMain.mnuMRU(I - 1).Visible = True
        Next I
        frmMain.mnuMRUsep.Visible = True
        
        ' Hide unused menu items
        If MRUFiles.Count < 4 Then
            For I = MRUFiles.Count + 1 To 3
                frmMain.mnuMRU(I - 1).Visible = False
            Next I
        End If
    Else
        For I = 0 To 3
            frmMain.mnuMRU(I).Visible = False
        Next
        frmMain.mnuMRUsep.Visible = False
    End If
End Sub

' Strips HTML from a string
Public Function StripHTML(HText$) As String
Dim T$
Dim is_tag%, Result$, I%
    ' Now scan the entire string to find the "<"'s and the ">"'s.
    T$ = HText$
    For I% = 1 To Len(T$)
       Select Case Mid$(T$, I%, 1)
          Case "<"
             is_tag% = True
          Case ">"
             is_tag% = False
          Case "&"
               Dim nt$
               nt = Mid$(T$, I%, 6)
               If LCase(nt) = "&nbsp;" Then
                   Result$ = Result$ & " "
                   I = I + 5
               ElseIf LCase(nt) = "&copy;" Then
                   I = I + 5
                   Result$ = Result$ & "(C)"
               ElseIf LCase(nt) = "&quot;" Then
                   I = I + 5
                   Result$ = Result$ & Chr$(34)
               ElseIf Left$(LCase(nt), 5) = "&amp;" Then
                   I = I + 4
                   Result$ = Result$ & "&"
               ElseIf Left$(LCase(nt), 4) = "&lt;" Then
                   I = I + 3
                   Result$ = Result$ & "<"
               ElseIf Left$(LCase(nt), 4) = "&gt;" Then
                   I = I + 3
                   Result$ = Result$ & ">"
               End If
          Case Else
             If Not is_tag% Then Result$ = Result$ & Mid$(T$, I%, 1)
       End Select
    Next
    
    StripHTML = Result$
End Function

' Converts characters <,>,", and & to HTML entities, when they occur outside of
' tags.
Function ConvertToEntities(HText$) As String
Dim T$
Dim is_tag As Boolean, Result$, I%
    ' Now scan the entire string to find the "<"'s and the ">"'s.
    T$ = HText$
    For I% = 1 To Len(T$)
       Select Case Mid$(T$, I%, 1)
          Case "<"
            If Mid$(T$, IIf(I = 1, 1, I - 1), 1) = " " And Mid$(T$, IIf(I = Len(T$), I, I + 1), 1) = " " Then
               Result$ = Result$ & "&lt;"
            Else
               is_tag = True
               Result$ = Result$ & Mid$(T$, I%, 1)
            End If
          Case ">"
             If Mid$(T$, IIf(I = 1, 1, I - 1), 1) = " " And Mid$(T$, IIf(I = Len(T$), I, I + 1), 1) = " " Then
                Result$ = Result$ & "&gt;"
             Else
                Result$ = Result$ & Mid$(T$, I%, 1)
                is_tag = False
             End If
          Case "&"
             If Not is_tag Then
                Result$ = Result$ & "&amp;"
             Else
                Result$ = Result$ & Mid$(T$, I%, 1)
             End If
          Case Chr$(34)
             If Not is_tag Then
                Result$ = Result$ & "&quot;"
             Else
                Result$ = Result$ & Mid$(T$, I%, 1)
             End If
          Case Else
             Result$ = Result$ & Mid$(T$, I%, 1)
       End Select
    Next
    
    ConvertToEntities = Result$
End Function

' Modify a string based on whether UCaseTags is true
Public Function CaseTag(Tag As String) As String
    CaseTag = IIf(UcaseTags, UCase(Tag), LCase(Tag))
End Function

' Surround a string in quotes (ASCII quotes, not HTML).
Public Function QuoteStr(Str$) As String
    QuoteStr = Chr$(34) & Str$ & Chr$(34)
End Function

'   This function returns the version of the pathname ToPath
' that is relative to FromPath. Relative pathnames are used
' when you want to access a file in another directory without
' using the full path, including drive letters.
'   Called from frmImageLink for use in IMG tags, but useful
' enough that it may be used elsewhere as well.
'
Public Function GetRelativePath(FromPath$, ToPath$) As String
Dim FPathDirLevels%, TPathDirLevels%, I%
Dim FPath$, TPath$, T$
Dim RPath$, N%
Dim StartDirLevel%

    FPath = GetFilePart(FromPath, kfpDrivePath)
    TPath = GetFilePart(ToPath, kfpDrivePath)
    T$ = GetFilePart(ToPath$, kfpFilename)
    
    ' Determine if the target file is in the same directory or a subdir of
    ' the base path.
    '
    If LCase(FPath) = LCase(TPath) Then
        GetRelativePath = T$
        Exit Function
    ElseIf LCase(TPath) Like (LCase(FPath) & "*") Then
        RPath$ = Mid$(TPath, Len(FPath) + 1, Len(TPath) - Len(FPath) + 1) & T$
        ' Replace backslashes w/forward ones
        For N = 1 To Len(RPath$)
            If Mid$(RPath, N, 1) = "\" Then Mid$(RPath, N, 1) = "/"
        Next N
        GetRelativePath = RPath$
        Exit Function
    End If
    
    ' Here we must conclude that the directories are on different branches
    ' of the folder tree.
    
    ' Get a count of the directory levels within the base path and the target
    ' path, beginning with the root dir = 1
    '
    For I = 1 To Len(FPath)
        If Mid$(FPath, I, 1) = "\" Then
            FPathDirLevels = FPathDirLevels + 1
        End If
    Next I
    For I = 1 To Len(TPath)
        If Mid$(TPath, I, 1) = "\" Then
            TPathDirLevels = TPathDirLevels + 1
        End If
    Next I
    
    ' Compare the two paths using ever-shrinking subdir levels
    
    ' Find the base comparison point (allowing for the fact that the two paths
    ' may have different numbers of subdir levels) and adjust the result
    ' string accordingly
    StartDirLevel = IIf(TPathDirLevels > FPathDirLevels, FPathDirLevels, _
        TPathDirLevels)
        
    If TPathDirLevels < FPathDirLevels Then
        For I = 1 To (FPathDirLevels - TPathDirLevels)
            RPath$ = RPath$ & "../"
        Next I
    End If
    
    ' Find the place where the two paths diverge.
    For I = StartDirLevel To 0 Step -1
        If LCase$(GetDirLevel(FPath, I)) <> LCase$(GetDirLevel(TPath, I)) Then
            RPath$ = RPath$ & "../"
        Else
            If I = 0 Then
                RPath$ = GetFilePart(TPath$, kfpPath)
            Else
                Dim tmp$
                tmp$ = Mid$(TPath, Len(GetDirLevel(TPath, I)), Len(TPath) - Len(GetDirLevel(TPath, I)) + 1)
                tmp = Right$(tmp, Len(tmp) - 1) 'nix first slash
                RPath$ = RPath$ & tmp$
            End If
            
            RPath$ = RPath$ & T$
            
            ' Replace backslashes w/forward ones
            For N = 1 To Len(RPath$)
                If Mid$(RPath, N, 1) = "\" Then Mid$(RPath, N, 1) = "/"
            Next N
            
            Exit For
        End If
    Next I
    
    
    GetRelativePath = RPath$
End Function

' This function aids the GetRelativePath function.  It returns an entire path up to
' the directory level specified; e.g., ("c:\data\hi\goodbye\",3) returns the
' string "c:\data\hi\"
'
Private Function GetDirLevel(Path$, dirlevel%) As String
Dim done As Boolean, I%, C%, Result$
    I = 1
    Do
        I = InStr(I, Path, "\")
        If I = 0 Then
            GetDirLevel = ""
            Exit Function
        Else
            C = C + 1
            If C = dirlevel Then
                GetDirLevel = Left$(Path, I)
                done = True
            End If
            I = I + 1
        End If
    Loop Until done
End Function
